<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:tei="http://www.tei-c.org/ns/1.0" xmlns:exist="http://exist-db.org/xquery/kwic" xmlns:xi="http://www.w3.org/2001/XInclude" xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs tei" version="2.0">
    
    <xsl:strip-space elements="tei:bibl tei:listBibl"/>
    
    <!-- ********** -->
    <!-- Variables  -->
    <!-- ********** -->
    
    <xsl:variable name="descr">description.html</xsl:variable> 
    <xsl:variable name="class">classification.html</xsl:variable>
    <xsl:variable name="taxonomy">../../data/grammateus_taxonomy.xml</xsl:variable>
    <xsl:variable name="authority">../../data/authority_lists.xml</xsl:variable>
    
    <!-- ********** -->
    <!-- Functions  -->
    <!-- ********** -->
    
    <!-- Extract the proper link in a <category> element from the <taxonomy>.
        Function use in for ref template, @type "descr" or "class"-->
    <xsl:function name="tei:link">
        <xsl:param name="type"/><!-- either "descr" or "class" -->
        <xsl:param name="category"/><!-- the xml:id of a <category> -->
        <xsl:value-of select="document($taxonomy)//tei:category[@xml:id eq $category]/tei:catDesc/tei:ptr[@type eq $type]/@target"/>
    </xsl:function>
    
    <!-- ********** -->
    <!-- Templates  -->
    <!-- ********** -->
    
    <xsl:template match="/">
        <div class="text">
        
            <!-- title/subtitle -->
            <h1 class="h1">
                <xsl:apply-templates select="//tei:body/tei:head[not(@type)]" mode="h1"/>
                <xsl:if test="tei:head[@type = 'subtitle']">
                    <!-- text-secondary is a bootstrap color class -->
                    <p class="subtitle text-secondary">
                        <xsl:apply-templates select="//tei:body/tei:head[@type = 'subtitle']" mode="h2"/>
                    </p>
                </xsl:if>
            </h1>
        
            <!-- contents -->
            <xsl:if test="//tei:body/tei:div[@type eq 'section']">
                <h3 class="h3">Contents</h3>
                <ol>
                    <xsl:for-each select="//tei:body/tei:div[@type eq 'section']">
                        <li>
                            <a href="#{./tei:head/@xml:id}">
                                <!-- tokenize the text to include only the first part of the heading and no greek text in parenthesis-->
                                <xsl:value-of select="./tei:head/tokenize(text()[1], ' \(')"/>
                            </a>
                            <xsl:if test="./tei:div[@type eq 'subsection']">
                                <ul>
                                    <xsl:for-each select="./tei:div[@type eq 'subsection']">
                                        <li>
                                            <a href="#{./tei:head/@xml:id}">
                                                <xsl:value-of select="./tei:head/tokenize(text()[1], ' \(')"/>
                                            </a>
                                        </li>
                                    </xsl:for-each>
                                </ul>
                            </xsl:if>
                        </li>
                    </xsl:for-each>
                </ol>
            </xsl:if>

            <!-- main text -->
            <xsl:apply-templates/>
            
            <!-- bibliography -->
            <xsl:if test="//tei:bibl">
                <h3 class="h3">Bibliography</h3>
                <ul>
                    <xsl:for-each-group select="//tei:bibl/tei:ptr" group-by="@target">
                        <xsl:sort select="@target"/>
                        <li>
                            <xsl:apply-templates select="."/>
                            <xsl:text>.</xsl:text>
                        </li>
                    </xsl:for-each-group>
                </ul>
            </xsl:if>
            
            <!-- how to cite -->
            <xsl:if test="//tei:idno[@type eq 'DOI']">
                <h3 class="h3">How to Cite</h3>
                <div class="pb-4">
                    <span>
                        <xsl:for-each select="document($authority)//tei:respStmt">
                            <xsl:sort select=".//tei:surname"/>
                            <xsl:if test="position() = last()">
                                <xsl:text>and </xsl:text>
                            </xsl:if>
                            <xsl:value-of select=".//tei:surname"/>
                            <xsl:text>, </xsl:text>
                            <xsl:value-of select="substring(.//tei:forename, 1, 1)"/>
                            <xsl:choose>
                                <xsl:when test="position() != last()">
                                    <xsl:text>., </xsl:text>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:text>. </xsl:text>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each>
                    </span>
                    <span class="title">
                        <xsl:value-of select="//tei:titleStmt/tei:title"/>
                        <xsl:text>. </xsl:text>
                    </span>
                    <span>grammateus project. </span>
                    <span>
                        <xsl:text>DOI: </xsl:text>
                        <xsl:variable name="doi" select="//tei:idno[@type eq 'DOI']"/>
                        <a href="https://doi.org/{$doi}">
                            <xsl:value-of select="$doi"/>
                     </a>
                    </span>
                </div>
            </xsl:if>
        </div>
    </xsl:template>
    
    
    <!-- general TEXT templates -->

    <xsl:template match="tei:head[not(@type)]" mode="h1">
       <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="tei:head[@type = 'subtitle']" mode="h2">
       <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="tei:div[@type = 'section']">
        <h3 class="h3">
            <a class="anchor" id="{tei:head/@xml:id}"/>
            <xsl:value-of select="tei:head"/>
        </h3>
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="tei:div[@type = 'subsection']">
        <h5 class="h5">
            <a class="anchor" id="{tei:head/@xml:id}"/>
            <xsl:value-of select="tei:head"/>
        </h5>
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="tei:p">
        <div class="paratext">
            <span class="paranumber">
                <xsl:value-of select="count(preceding::tei:p[ancestor::tei:body])+1"/>
            </span>
            <p>
                <xsl:if test="@xml:id">
                    <a class="anchor" id="{@xml:id}"/>
                </xsl:if>
                <xsl:apply-templates/>
            </p>
        </div>
    </xsl:template>
    
    <xsl:template match="tei:list[@type = 'ordered']">
        <ol>
            <xsl:attribute name="type">
                <xsl:value-of select="tei:item[1]/@n"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </ol>
    </xsl:template>
    
    <xsl:template match="tei:list[@type = 'unordered']">
        <ul>
            <xsl:apply-templates/>
        </ul>
    </xsl:template>
    
    <xsl:template match="tei:item">
        <li>
            <xsl:apply-templates/>
        </li>
    </xsl:template>
    
    <xsl:template match="tei:title[@rend = 'italic']">
        <cite>
            <xsl:apply-templates/>
        </cite>
    </xsl:template>
    
    <xsl:template match="tei:hi[@rend = 'italic']">
        <i>
            <xsl:apply-templates/>
        </i>
    </xsl:template>
    
    <xsl:template match="tei:hi[@rend = 'bold']">
        <b>
            <xsl:apply-templates/>
        </b>
    </xsl:template>
    
    <xsl:template match="tei:q[not(parent::tei:q)]">
        <xsl:text>“</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>”</xsl:text>
    </xsl:template>
    
    <xsl:template match="tei:q[parent::tei:q]">
        <xsl:text>‘</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>’</xsl:text>
    </xsl:template>
    
    <xsl:template match="tei:term[@xml:lang = 'grc']">
        <span class="grc">
            <xsl:apply-templates/>
        </span>
    </xsl:template>
    
    <xsl:template match="tei:term[@xml:lang = 'lat']">
        <span class="lat" style="font-style:italic;">
            <xsl:apply-templates/>
        </span>
    </xsl:template>
    
    
    <!-- POINTERS and LINKS -->
    
    <!-- pointer for cross-references (list items) -->
    <xsl:template match="tei:ptr[@type = 'list']">
        <xsl:variable name="target" select="@target"/>
        <xsl:value-of select="ancestor::tei:div//tei:item[@xml:id = substring-after($target, '#')]/@n"/>)
    </xsl:template>
    
    <!-- links to description page -->
    <xsl:template match="tei:ref[@type = 'descr']">
        <a href="{$descr}?d={tei:link(@type, substring-after(@target, '#'))}{tei:ptr/@target}">
            <xsl:apply-templates/>
        </a>
    </xsl:template>
    
    <!-- links to classification page -->
    <xsl:template match="tei:ref[@type = 'class']">
        <a href="{$class}?d={tei:link(@type, substring-after(@target, '#'))}{tei:ptr/@target}">
            <xsl:apply-templates/>
        </a>
    </xsl:template>
    
    <!-- anchor for links in same document -->
    <xsl:template match="tei:ref[@type = 'anchor']">
        <a href="{@target}">
            <xsl:apply-templates/>
        </a>
    </xsl:template>
    
    <!-- links to papyri not in our corpus / in papyri.info -->
    <xsl:template match="tei:ref[@type = 'external']">
        <a class="out-link" href="{@target}" target="_blank">
            <xsl:apply-templates/>
        </a>
    </xsl:template>
    
    <!-- links to papyri available in our corpus -->
    <xsl:template match="tei:ref[@type = 'internal']">
        <a class="db-link" href="show.html?document={@target}" target="_blank">
            <xsl:apply-templates/>
        </a>
    </xsl:template>
    
    <!-- other links -->
    <xsl:template match="tei:ref[not(@type) or @type eq 'other']">
        <a href="{@target}" target="_blank">
            <xsl:apply-templates/>
        </a>
    </xsl:template>
    
    
    <!-- BIBLIOGRAPHY templates -->
    
    <!-- Inline citations: links to our own bibliography page -->
    <xsl:template match="tei:bibl">
        <xsl:text>[</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>]</xsl:text>
    </xsl:template>
    
    <xsl:template match="tei:listBibl">
        <xsl:text>[</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>]</xsl:text>
    </xsl:template>
    
    <xsl:template match="tei:bibl[parent::tei:listBibl]">
        <xsl:apply-templates/>
        <xsl:if test="position() != last()">
            <xsl:text>; </xsl:text>  
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="tei:ptr[parent::tei:bibl]">
        <!-- path to bibliographic file from ptr @target -->
        <xsl:variable name="path">
            <xsl:text>../../data/bibl/</xsl:text>
            <xsl:value-of select="substring-after(@target, 'gramm:')"/>
            <xsl:text>.xml</xsl:text>
        </xsl:variable>
        <!-- get bibliographic file  -->
        <xsl:variable name="bibl-doc" select="document($path)"/>
        <!-- author nb -->
        <xsl:variable name="author" select="count($bibl-doc//tei:text//tei:author)"/>
        <!-- editor nb -->
        <xsl:variable name="editor" select="count($bibl-doc//tei:text//tei:editor)"/>
        <!-- special short title instead of author (for Papyri editions like Paramone) -->
        <xsl:variable name="short-title" select="$bibl-doc//tei:title[@type eq 'short' and @level eq 'm']"/>
        <!-- link to our bibliography -->
        <a href="bibliography.html#{substring-after(@target, 'gramm:')}" target="_blank">
            <!-- display author depending on author/editor nb -->
            <xsl:choose>
                <xsl:when test="$short-title">
                    <xsl:value-of select="$short-title"/>
                </xsl:when>
                <xsl:when test="$author = 1">
                    <xsl:value-of select="$bibl-doc//tei:author/tei:surname/string()"/>
                </xsl:when>
                <xsl:when test="$author = 2">
                    <xsl:value-of select="$bibl-doc//tei:author[1]/tei:surname/string()"/>
                    <xsl:text> and </xsl:text>
                    <xsl:value-of select="$bibl-doc//tei:author[2]/tei:surname/string()"/>
                </xsl:when>
                <xsl:when test="$author gt 2">
                    <xsl:value-of select="$bibl-doc//tei:author[1]/tei:surname/string()"/>
                    <xsl:text> </xsl:text>
                    <i>et al.</i>
                </xsl:when>
                <!-- no author -->
                <xsl:when test="$editor = 1">
                    <xsl:value-of select="$bibl-doc//tei:editor/tei:surname/string()"/>
                </xsl:when>
                <xsl:when test="$editor = 2">
                    <xsl:value-of select="$bibl-doc//tei:editor[1]/tei:surname/string()"/>
                    <xsl:text> and </xsl:text>
                    <xsl:value-of select="$bibl-doc//tei:editor[2]/tei:surname/string()"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$bibl-doc//tei:editor[1]/tei:surname/string()"/>
                    <xsl:text> </xsl:text>
                    <i>et al.</i>
                </xsl:otherwise>
            </xsl:choose>
        </a>
        <!-- date -->
        <xsl:choose>
            <xsl:when test="$short-title"/>
            <xsl:otherwise>
                <xsl:text> </xsl:text>
                <xsl:value-of select="$bibl-doc//tei:imprint/tei:date/substring(text(), 1, 4)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="tei:citedRange">
        <xsl:text> : </xsl:text>
        <xsl:apply-templates/>
    </xsl:template>
    
    <!-- templates for the expanded full text search results? Not working. -->
    <xsl:template match="exist:match">
        <exist:match>
            <xsl:apply-templates/>
        </exist:match>
    </xsl:template>
    
    
    <!-- TABLES templates -->
    
    <xsl:template match="tei:table">
        <table>
            <xsl:apply-templates/>
        </table>
    </xsl:template>
    
    <xsl:template match="tei:row">
        <tr>
            <xsl:apply-templates/>
        </tr>
    </xsl:template>
    
    <xsl:template match="tei:row[@role eq 'label']">
        <thead>
            <tr>
                <xsl:apply-templates/>
            </tr>
        </thead>
    </xsl:template>
    
    <xsl:template match="tei:cell[parent::tei:row[@role eq 'label']]">
        <th>
            <xsl:apply-templates/>
        </th>
    </xsl:template>
    
    <xsl:template match="tei:cell[@rows]">
        <td rowspan="{@rows}">
            <xsl:apply-templates/>
        </td>
    </xsl:template>
    
    <xsl:template match="tei:cell">
        <td>
            <xsl:apply-templates/>
        </td>
    </xsl:template>
    
    
    <!-- ********************** -->
    <!-- elements not processed -->
    <xsl:template match="tei:teiHeader"/>
    <xsl:template match="tei:head"/>
    <xsl:template match="tei:catDesc"/>
    <xsl:template match="tei:author"/>
    
</xsl:stylesheet>